==> e2fsprogs-1.17.tar.gz: e2fsprogs-1.17/misc/lsattr.c <==
/*
 * lsattr.c		- List file attributes on an ext2 file system
 *
 * Copyright (C) 1993, 1994  Remy Card <card@masi.ibp.fr>
 *                           Laboratoire MASI, Institut Blaise Pascal
 *                           Universite Pierre et Marie Curie (Paris VI)
 *
 * This file can be redistributed under the terms of the GNU General
 * Public License
 */

/*
 * History:
 * 93/10/30	- Creation
 * 93/11/13	- Replace stat() calls by lstat() to avoid loops
 * 94/02/27	- Integrated in Ted's distribution
 * 98/12/29	- Display version info only when -V specified (G M Sipe)
 */

#include <sys/types.h>
#include <dirent.h>
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#include <fcntl.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
extern int optind;
extern char *optarg;
#endif
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <linux/ext2_fs.h>

#include "et/com_err.h"
#include "e2p/e2p.h"

#include "../version.h"

const char * program_name = "lsattr";

int all = 0;
int d_opt = 0;
int l_opt = 0;
int recursive = 0;
int verbose = 0;
int v_opt = 0;

static void usage(void)
{
	fprintf(stderr, "Usage: %s [-RVadlv] [files...]\n", program_name);
	exit(1);
}

static void list_attributes (const char * name)
{
	unsigned long flags;
	unsigned long version;

	if (fgetflags (name, &flags) == -1)
		com_err (program_name, errno, "While reading flags on %s",
			 name);
	else if (fgetversion (name, &version) == -1)
		com_err (program_name, errno, "While reading version on %s",
			 name);
	else
	{
		if (v_opt)
			printf ("%5lu ", version);
		print_flags (stdout, flags, l_opt);
		printf (" %s\n", name);
	}
}

static int lsattr_dir_proc (const char *, struct dirent *, void *);

static void lsattr_args (const char * name)
{
	struct stat st;

	if (lstat (name, &st) == -1)
		com_err (program_name, errno, "while stating %s", name);
	else
	{
		if (S_ISDIR(st.st_mode) && !d_opt)
			iterate_on_dir (name, lsattr_dir_proc, (void *) NULL);
		else
			list_attributes (name);
	}
}

static int lsattr_dir_proc (const char * dir_name, struct dirent * de, void * private)
{
	struct stat st;
	char *path;

	path = malloc(strlen (dir_name) + 1 + strlen (de->d_name) + 1);

	sprintf (path, "%s/%s", dir_name, de->d_name);
	if (lstat (path, &st) == -1)
		perror (path);
	else {
		if (de->d_name[0] != '.' || all) {
			list_attributes (path);
			if (S_ISDIR(st.st_mode) && recursive &&
			    strcmp(de->d_name, ".") &&
			    strcmp(de->d_name, "..")) {
				printf ("\n%s:\n", path);
				iterate_on_dir (path, lsattr_dir_proc,
						(void *) NULL);
				printf ("\n");
			}
		}
	}
	free(path);
	return 0;
}

int main (int argc, char ** argv)
{
	int c;
	int i;

	if (argc && *argv)
		program_name = *argv;
	while ((c = getopt (argc, argv, "RVadlv")) != EOF)
		switch (c)
		{
			case 'R':
				recursive = 1;
				break;
			case 'V':
				verbose = 1;
				break;
			case 'a':
				all = 1;
				break;
			case 'd':
				d_opt = 1;
				break;
			case 'l':
				l_opt = 1;
				break;
			case 'v':
				v_opt = 1;
				break;
			default:
				usage();
		}

	if (verbose)
		fprintf (stderr, "lsattr %s, %s for EXT2 FS %s, %s\n",
			 E2FSPROGS_VERSION, E2FSPROGS_DATE,
			 EXT2FS_VERSION, EXT2FS_DATE);
	if (optind > argc - 1)
		lsattr_args (".");
	else
		for (i = optind; i < argc; i++)
			lsattr_args (argv[i]);
	exit(0);
}

==> e2fsprogs-1.19.tar.gz: e2fsprogs-1.19/misc/lsattr.c <==
/*
 * lsattr.c		- List file attributes on an ext2 file system
 *
 * Copyright (C) 1993, 1994  Remy Card <card@masi.ibp.fr>
 *                           Laboratoire MASI, Institut Blaise Pascal
 *                           Universite Pierre et Marie Curie (Paris VI)
 *
 * This file can be redistributed under the terms of the GNU General
 * Public License
 */

/*
 * History:
 * 93/10/30	- Creation
 * 93/11/13	- Replace stat() calls by lstat() to avoid loops
 * 94/02/27	- Integrated in Ted's distribution
 * 98/12/29	- Display version info only when -V specified (G M Sipe)
 */

#include <sys/types.h>
#include <dirent.h>
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#include <fcntl.h>
#ifdef HAVE_GETOPT_H
#include <getopt.h>
#else
extern int optind;
extern char *optarg;
#endif
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <linux/ext2_fs.h>

#include "et/com_err.h"
#include "e2p/e2p.h"

#include "../version.h"
#include "nls-enable.h"

static const char * program_name = "lsattr";

static int all = 0;
static int dirs_opt = 0;
static unsigned pf_options = 0;
static int recursive = 0;
static int verbose = 0;
static int generation_opt = 0;

static void usage(void)
{
	fprintf(stderr, _("Usage: %s [-RVadlv] [files...]\n"), program_name);
	exit(1);
}

static void list_attributes (const char * name)
{
	unsigned long flags;
	unsigned long generation;

	if (fgetflags (name, &flags) == -1)
		com_err (program_name, errno, _("While reading flags on %s"),
			 name);
	else if (fgetversion (name, &generation) == -1)
		com_err (program_name, errno, _("While reading version on %s"),
			 name);
	else
	{
		if (generation_opt)
			printf ("%5lu ", generation);
		if (pf_options & PFOPT_LONG) {
			printf("%-28s ", name);
			print_flags(stdout, flags, pf_options);
			fputc('\n', stdout);
		} else {
			print_flags(stdout, flags, pf_options);
			printf(" %s\n", name);
		}
	}
}

static int lsattr_dir_proc (const char *, struct dirent *, void *);

static void lsattr_args (const char * name)
{
	struct stat st;

	if (lstat (name, &st) == -1)
		com_err (program_name, errno, _("while trying to stat %s"),
			 name);
	else {
		if (S_ISDIR(st.st_mode) && !dirs_opt)
			iterate_on_dir (name, lsattr_dir_proc, NULL);
		else
			list_attributes (name);
	}
}

static int lsattr_dir_proc (const char * dir_name, struct dirent * de, void * private)
{
	struct stat st;
	char *path;

	path = malloc(strlen (dir_name) + 1 + strlen (de->d_name) + 1);

	sprintf (path, "%s/%s", dir_name, de->d_name);
	if (lstat (path, &st) == -1)
		perror (path);
	else {
		if (de->d_name[0] != '.' || all) {
			list_attributes (path);
			if (S_ISDIR(st.st_mode) && recursive &&
			    strcmp(de->d_name, ".") &&
			    strcmp(de->d_name, "..")) {
				printf ("\n%s:\n", path);
				iterate_on_dir (path, lsattr_dir_proc, NULL);
				printf ("\n");
			}
		}
	}
	free(path);
	return 0;
}

int main (int argc, char ** argv)
{
	int c;
	int i;

#ifdef ENABLE_NLS
	setlocale(LC_MESSAGES, "");
	bindtextdomain(NLS_CAT_NAME, LOCALEDIR);
	textdomain(NLS_CAT_NAME);
#endif
	if (argc && *argv)
		program_name = *argv;
	while ((c = getopt (argc, argv, "RVadlv")) != EOF)
		switch (c)
		{
			case 'R':
				recursive = 1;
				break;
			case 'V':
				verbose = 1;
				break;
			case 'a':
				all = 1;
				break;
			case 'd':
				dirs_opt = 1;
				break;
			case 'l':
				pf_options = PFOPT_LONG;
				break;
			case 'v':
				generation_opt = 1;
				break;
			default:
				usage();
		}

	if (verbose)
		fprintf (stderr, _("lsattr %s, %s for EXT2 FS %s, %s\n"),
			 E2FSPROGS_VERSION, E2FSPROGS_DATE,
			 EXT2FS_VERSION, EXT2FS_DATE);
	if (optind > argc - 1)
		lsattr_args (".");
	else
		for (i = optind; i < argc; i++)
			lsattr_args (argv[i]);
	exit(0);
}
